const { SLAVE_GENERATION_NUMBER } = require('../entity/slave');
const { exec } = require('child_process');
const { slaves } = require('../entity/slave');

function initSwarmManager() {
    console.log("Creating ");
    exec("docker swarm leave --force", (err, stdout, stderr) => {
        if (err) {
            console.error(`error: ${err}`);
        }
    })
    exec("docker swarm init --advertise-addr 127.0.0.1", (err, stdout, stderr) => {
        if (err) {
            console.error(`error: ${err}`);
        } else {
            console.log("Creating slaves using replicas");
            exec("docker service create --restart-condition='none' --network='host' --name slaves --replicas " + SLAVE_GENERATION_NUMBER + " servuc/hash_extractor ./hash_extractor s ws://localhost:3001/slaves", (err, stdout, stderr) => {
                if (err) {
                    console.error(`error: ${err}`);
                } else {
                    setInterval(checkInactive, 5000);
                }
            });
        }
    });
}

function checkInactive() {
    let inactifs = getInactiveSlaves();

    if (inactifs.length < SLAVE_GENERATION_NUMBER)
        scaleSlaves(SLAVE_GENERATION_NUMBER - inactifs.length);
    else {
        for (let i = 0; i < inactifs.length - SLAVE_GENERATION_NUMBER; i++) {
            let slave = inactifs[i];
            console.log("Shutdown slave : " + slave.name);
            slave.ws.send("exit");
            slaves = slaves.filter(s => s !== slave)
        }
    }
}

function scaleSlaves(nbSlavesManquant) {
    let slavesTot = slaves.length + nbSlavesManquant;
    exec("docker service scale slaves=" + slavesTot, (err, _stdout, _stderr) => {
        if (err) {
            console.error(`error: ${err}`);
        } else {
            console.log("Scale slaves to " + slavesTot);
        }
    });
}


function stopSearch(hashCurrent, hash) {
    if (hash in hashCurrent) {
        for (let slave of hashCurrent[hash]) {
            slave.ws.send("stop");
            slave.ws.send("exit");
        }
        delete hashCurrent[hash];
    }
}

function getInactiveSlaves() {
    return slaves.filter(s => !s.active);
}

module.exports = { initSwarmManager, checkInactive, scaleSlaves, stopSearch, getInactiveSlaves }